/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Helper class for logging
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Logger.hxx,v 1.2 2008/11/17 10:20:14 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_CONF_SERVICE_LOGGER_HXX_
#define GLITE_DATA_CONF_SERVICE_LOGGER_HXX_

#include <string>
#include <log4cpp/Category.hh>

namespace glite {
namespace config {

/**
 * Utility class for logging
 */
class Logger {
public:
    /**
     * constructor
     * @param category [IN] the name of the logger category
     */
    Logger(const std::string& category) : 
        m_logger(log4cpp::Category::getInstance(category)),
        m_categoryName(category){
    }
    
    /**
     * copy constructor
     * @param other [IN]
     * NOTE: initializing the logger reference with other.m_logger causes a crash
     * (probably due to different template member functions instantiation across
     * translation units)
     */
    Logger(const Logger& other) : 
        m_logger(log4cpp::Category::getInstance(other.m_categoryName)),
        m_categoryName(other.m_categoryName){
    }
    
    /**
     * destructor
     */
    ~Logger(){
    }
    
    /**
     * get the logging category
     */
    log4cpp::Category& logger() const{
        return m_logger;
    }

private:

    // the logging category
    mutable log4cpp::Category& m_logger;
    
    // the category name
    std::string m_categoryName;
    
};

} // namespace config
} // namespace glite

//------------------------------------------------------------------------------
//                              utility functions
//------------------------------------------------------------------------------
namespace{
    
const char * str_bool(bool value){
    return value ? "true" : "false";
}
const char * str_enabled(bool value){
    return value ? "enabled" : "disabled";
}
const char * str_char(char * value){
    return value ? value : "(null)";
}

}

//------------------------------------------------------------------------------
//                                  helper macros 
//------------------------------------------------------------------------------

// log a debug message only in debug builds
#ifdef _DEBUG
#define m_log_trace(msg) m_logger.logger().debugStream() << "[TRACE] " << msg
#define _log_trace(cat,msg) log4cpp::Category::getInstance(cat).debugStream() << "[TRACE] " << msg
#else
#define m_log_trace(msg)
#define _log_trace(cat,msg)
#endif

// log a debug message
#define m_log_debug(msg) m_logger.logger().debugStream() << msg
#define _log_debug(cat,msg) log4cpp::Category::getInstance(cat).debugStream() << msg
// log an informational message
#define m_log_info(msg) m_logger.logger().infoStream() << msg
#define _log_info(cat,msg) log4cpp::Category::getInstance(cat).infoStream() << msg
// log a notice message
#define m_log_notice(msg) m_logger.logger().noticeStream() << msg
#define _log_notice(cat,msg) log4cpp::Category::getInstance(cat).noticeStream() << msg
// log a warning message
#define m_log_warn(msg) m_logger.logger().warnStream() << msg
#define _log_warn(cat,msg) log4cpp::Category::getInstance(cat).warnStream() << msg
// log an error message
#define m_log_error(msg) m_logger.logger().errorStream() << msg
#define _log_error(cat,msg) log4cpp::Category::getInstance(cat).errorStream() << msg
// log a critical message
#define m_log_crit(msg) m_logger.logger().critStream() << msg
#define _log_crit(cat,msg) log4cpp::Category::getInstance(cat).critStream() << msg
// log an alert
#define m_log_alert(msg) m_logger.logger().alertStream() << msg
#define _log_alert(cat,msg) log4cpp::Category::getInstance(cat).alertStream() << msg
// log a fatal error
#define m_log_fatal(msg) m_logger.logger().fatalStream() << msg
#define _log_fatal(cat,msg) log4cpp::Category::getInstance(cat).fatalStream() << msg

#endif // GLITE_DATA_CONF_SERVICE_LOGGER_HXX_


