/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Config Service : configuration helpers
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ParamReader.hxx,v 1.5 2008/12/17 13:39:15 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_CONF_SERVICE_PARAMREADER_HXX_
#define GLITE_DATA_CONF_SERVICE_PARAMREADER_HXX_

#include <glite/data/config/service/ParamValue.hxx>
#include <glite/data/config/service/ServiceConfigurationException.hxx>
#include <glite/data/config/service/StringConverter.hxx>

#include <string>

namespace glite {
namespace config{

/**
 * ParamReader
 * Helper class to read configuration parameters.
 * @todo move to config-service
 */
class ParamReader{
public:
    
    /**
     * Parameter validator.
     */
    template<typename ParamValueType>
    class ParamValidator{
    public:
        
        /**
         * required
         * throw an exception if parameter is missing
         * @return 
         */
        ParamValidator& required(){
            if (!m_init){
                throw MissingParamException(m_componentName,m_paramName);
            }
            return (*this);
        }

        /**
         * check
         * throw an exception if condition is false
         * @param condition 
         * @return 
         */
        ParamValidator& check(bool condition) __attribute__((deprecated)){
            if (m_init && !condition){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        
        ParamValidator& eq(ParamValueType value){
            if (m_init && (m_value != value)){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        ParamValidator& ne(ParamValueType value){
            if (m_init && (m_value == value)){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        ParamValidator& gt(ParamValueType value){
            if (m_init && (m_value <= value)){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        ParamValidator& ge(ParamValueType value){
            if (m_init && (m_value < value)){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        ParamValidator& lt(ParamValueType value){
            if (m_init && (m_value >= value)){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        ParamValidator& le(ParamValueType value){
            if (m_init && (m_value > value)){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        ParamValidator& not_empty(){
            if (m_init && m_value.empty()){
                throw InvalidParamValueException(m_componentName,m_paramName);
            }
            return (*this);
        }
        
    private:
        
        friend class ParamReader;
        
        /**
         * constructor
         * @param componentName 
         * @param paramName 
         * @param value 
         * @return 
         */
        ParamValidator(
            const std::string& componentName,
            const std::string& paramName,
            const ParamValueType value) :
            m_init(true),
            m_componentName(componentName),
            m_paramName(paramName),
            m_value(value){
        }

        /**
         * constructor
         * @param componentName 
         * @param paramName 
         * @return 
         */
        ParamValidator(
            const std::string& componentName,
            const std::string& paramName) :
            m_init(false),
            m_componentName(componentName),
            m_paramName(paramName){
        }
        
        // initialization flag
        bool m_init;
        // component name
        std::string m_componentName;
        // parameter name
        std::string m_paramName;
        // parameter value
        ParamValueType m_value;
    };
    
    /**
     * constructor
     * @param componentName 
     * @param params 
     * @return 
     */
    ParamReader(
        const std::string&                                      componentName,
        const glite::config::ComponentConfiguration::Params&    params) : 
        m_componentName(componentName),
        m_params(params){
    }
    
    /**
     * getParameter
     * get value for parameter with given name and return a validator for it.
     * @param name [IN] the parameter name
     * @param value [OUT] the parameter value
     * @return ParamValidator object
     */
    template<typename ParamValueType>
    ParamValidator<ParamValueType> getParameter(
        const std::string&  paramName,
        ParamValueType&     value){
        ComponentConfiguration::Params::const_iterator it = m_params.find(paramName);
        if (m_params.end() != it){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if (0 == pv){
                throw InvalidParamValueException(m_componentName,paramName);
            }
            value = StringConverter<ParamValueType>::from_string(pv->getValue());
            return ParamValidator<ParamValueType>(m_componentName,paramName,value);
        } else {
            return ParamValidator<ParamValueType>(m_componentName,paramName);
        }
    }

    /**
     * isSet
     * @param paramName [IN] the parameter name
     * @return true if parameter is present
     */
    bool isSet(const std::string& paramName){
        ComponentConfiguration::Params::const_iterator it = m_params.find(paramName);
        return (m_params.end() != it);
    }
    
    /**
     * getValue
     * @param paramName [IN] the parameter name
     * @return parameter value
     */
    template<typename ParamValueType>
    ParamValueType getValue(const std::string& paramName){
        ComponentConfiguration::Params::const_iterator it = m_params.find(paramName);
        if (m_params.end() == it){
            throw MissingParamException(m_componentName,paramName);
        }
        ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
        if (0 == pv){
            throw InvalidParamValueException(m_componentName,paramName);
        }
        return StringConverter<ParamValueType>::from_string(pv->getValue());
    }
        
protected:
    
    // component name
    std::string m_componentName;
    // list of parameters
    const glite::config::ComponentConfiguration::Params& m_params;
};

} // namespace config
} // namespace glite

#endif // GLITE_DATA_CONF_SERVICE_PARAMREADER_HXX_

