/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Config Service : Exception classes
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ServiceConfigurationException.hxx,v 1.2 2008/06/10 09:48:31 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_CONFIG_SERVICECONFIGURATIONEXCEPTION_HXX_
#define GLITE_DATA_CONFIG_SERVICECONFIGURATIONEXCEPTION_HXX_
  
#include <string>
#include <stdexcept>

namespace glite {
namespace config{   

/**
 * ServiceConfigurationException
 * a configuration exception
 */
class ServiceConfigurationException : public std::runtime_error{
public:
    /**
     * Constructor
     * @param reason [IN] exception description
     */
    explicit ServiceConfigurationException(
        const std::string& reason) : std::runtime_error(reason){}
    /**
     * Destructor
     */
    virtual ~ServiceConfigurationException() throw(){}
private:
    // Default constructor not implemented
    ServiceConfigurationException();
};

/**
 * ParamExceptionInfo
 * component and parameter name
 */
class ParamExceptionInfo{
public:
    /**
     * componentName
     * @return the name of the component
     */
    const std::string& componentName() const {
        return m_componentName;
    }
    /**
     * paramName
     * @return the name of the missing parameter
     */
    const std::string& paramName() const {
        return m_paramName;
    }
protected:
    /**
     * Constructor
     * @param componentName [IN]
     * @param paramName [IN] 
     */
    ParamExceptionInfo(
        const std::string& componentName,
        const std::string& paramName) : 
        m_componentName(componentName),
        m_paramName(paramName){
    }
private:
    // component name
    std::string m_componentName;
    // param name
    std::string m_paramName;
    // Default constructor not implemented
    ParamExceptionInfo();         
};

/**
 * MissingParamException
 * a missing parameter exception
 */
class MissingParamException : 
    public ParamExceptionInfo,
    public ServiceConfigurationException{
public:
    /**
     * Constructor
     * @param componentName [IN] the name of the component
     * @param paramName [IN] the name of the missing parameter
     */
    MissingParamException(
        const std::string& componentName,
        const std::string& paramName) : 
        ParamExceptionInfo(componentName,paramName),
        ServiceConfigurationException(std::string("Missing mandatory parameter ") + paramName + std::string(" in component ") + componentName){        
    }
    /**
     * Destructor
     */
    virtual ~MissingParamException() throw(){}
private:    
    // Default constructor not implemented
    MissingParamException();
};

/**
 * MissingParamException
 * an invalid parameter value exception
 */
class InvalidParamValueException : 
    public ParamExceptionInfo,
    public ServiceConfigurationException{
public:
    /**
     * Constructor
     * @param componentName [IN] the name of the component
     * @param paramName [IN] the name of the missing parameter
     */
    InvalidParamValueException(
        const std::string& componentName,
        const std::string& paramName) : 
        ParamExceptionInfo(componentName,paramName),
        ServiceConfigurationException(std::string("Invalid value for parameter ") + paramName + std::string(" in component ") + componentName){        
    }
    /**
     * Destructor
     */
    virtual ~InvalidParamValueException() throw(){}
private:
    // Default constructor not implemented
    InvalidParamValueException();
};

} // namespace config
} // namespace glite

#endif // GLITE_DATA_CONFIG_SERVICECONFIGURATIONEXCEPTION_HXX_

