/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ServiceConfiguration.hxx
 *
 * Authors: Paolo Badino <paolo.badino@cern.ch>
 * Version info: $Id: ServiceConfigurator.hxx,v 1.15 2008/07/02 08:25:41 ptedesco Exp $
 * Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_CONFIG_SERVICE_SERVICECONFIGURATOR_HXX_
#define GLITE_DATA_CONFIG_SERVICE_SERVICECONFIGURATOR_HXX_

#include <string>
#include <map>
#include <list>
#include "glite/data/config/service/ComponentConfiguration.hxx"

namespace log4cpp{
    class Category;
}

namespace glite {
namespace config{

class Locations;    
    
/**
 * ServiceConfigurator
 * this class read the configuration from a XML file and then dinamically loads
 * and configure all the Components that should compose the Service Application 
 */
class ServiceConfigurator {
public:

    /**
     * ServiceConfigurator return values
     */
    enum {
        SC_FAILED           = -1,
        SC_NOT_INIT         = -2,
        SC_NOT_STARTED      = -3,
        SC_SUCCESS          =  0,
        SC_ALREADY_INIT     =  1,
        SC_ALREADY_STARTED  =  2
    };
    
    /**
     * Return Singleton instance
     * @return Singleton instance
     */
    static ServiceConfigurator * instance();

    /**
     * Deletes singleton instance. If the service is running, it will be stopped
     * and finalized.
     */
    static void dispose();

    /**
     * Initialize the Service
     * @param locations [IN] service information
     * @return SC_SUCCESS if success, SC_ALREADY_INIT if already initialized or 
     * SC_FAILED if failed
     */
    int init(const Locations * locations);

    /**
     * Start the Service
     * @return SC_SUCCESS if success, SC_ALREADY_STARTED if already started,
     * SC_NOT_INIT if not initialized or SC_FAILED if failed
     */
    int start();

    /**
     * Stop the Service
     * @return SC_SUCCESS if success, SC_NOT_STARTED if not started, SC_NOT_INIT 
     * if not initialized or SC_FAILED if failed
     */
    int stop();
    
    /**
     * Reconfigure the Service. This method will perform the action 
     * stop-config-start on all the components
     * @return SC_SUCCESS if success, SC_NOT_INIT if not initialized or 
     * SC_FAILED if failed
     */
    int reconfig();
    
    /**
     * Finalize the Service
     * @return SC_SUCCESS if success, SC_NOT_INIT if not initialized or 
     * SC_FAILED if failed
     */
    int fini();

    /**
     * Register a Component: it will be configured without loading it dynamically
     * @param component [IN] the component to be registered
     * @return 0 if success
     */
    int registerComponent(ComponentConfiguration * component);

    /**
     * Deregister a Static Component
     * @param component [IN] the component to be deregistered
     * @return 0 if success
     */
    int deregisterComponent(ComponentConfiguration * component);
    
    /**
     * Initialize variables for changing the process name. This method should be
     * called in the main function if the process is interested in forking child
     * processing and updating their names
     * @param argc [IN] the number of command line arguments
     * @param argv [IN] the command line arguments
     * @return the maximum length of the name that could stored
     */
    static int initProcNameVars(int argc,char **argv);

    /**
     * Change the process name
     * @param name [IN] the name to assign to the process
     * @return the length of the name assigned to the process
     */
    static int setProcName(const char * name);
    
protected:

    /**
     * Constructor
     * @param name [IN] the name of the service
     */
    ServiceConfigurator();
    
    /**
     * Destructor
     */
    ~ServiceConfigurator();
    
    /**
     * Load Components
     * @return 0 if success
     */
    int loadComponents();

    /**
     * Reload Configuration
     * @return 0 if success
     */
    int reloadConfiguration();
    
    /**
     * Initialize Components
     * @return 0 if success
     */
    int initComponents();

    /**
     * Initialize Logger
     * @return 0 if success
     */
    int initLogger();

private:
    
    /**
     * ComponentItem Structure
     */
    struct ComponentItem {
        typedef std::list<void*>        HandleList;
        
        void *                          comp_handle;
        ComponentConfiguration *        component;
        HandleList                      dependencies;
        ComponentConfiguration::Params  init_params;
        ComponentConfiguration::Params  config_params;
    };
    
    typedef std::map<std::string, ComponentItem *> ComponentMap;
    typedef std::list<ComponentItem *>             ComponentList;
    
    /**
     * Dynamically load a Component
     * @param name      [IN] component name
     * @param libname   [IN] name of the component library
     * @param deplist   [IN] list of dependencies (names of libraries to open 
     * with dlopen)
     * @return pointer to ComponentItem object if successful, 0 otherwise
     */
    ComponentItem * loadComponent(
        const char *                    name, 
        const char *                    libname, 
        const std::list<std::string>&   deplist);
    
    /*
     * validate instance name
     */
    bool validateName(const char* name);
    
    /**
     * log_error_message
     * log an error message to the default logger and also to the file
     * /tmp/<svc-name>-svc-conf-debug.log in case the logger is not yet 
     * configured.
     * @param msg the message to log
     */
    void log_error_message(const char * msg);
            
    /**
     * log_debug_message
     * log a debug message to the default logger and also to the file
     * /tmp/<svc-name>-svc-conf-debug.log in case the logger is not yet 
     * configured.
     * @param msg the message to log
     */
    void log_debug_message(const char * msg);
    
private:

    typedef enum {
        SC_STATUS_IDLE          = 0,
        SC_STATUS_INITIALIZED   = 1,
        SC_STATUS_STARTED       = 2,
        SC_STATUS_FAILED        = 3
    } SCStatus;
    
    /**
     * Service Name
     */
    std::string m_name;    

    /**
     * Service Directory Name
     */
    std::string m_dirName;    
    
    /**
     * Service Instance Name
     */
    std::string m_instanceName;    
    
    /**
     * Service Status
     */
    SCStatus m_status;

    /**
     * The Service Component Map
     */
    ComponentMap m_components;
    
    /**
     * The Service Component List
     */
    ComponentList m_componentList;
    
    /**
     * The Logger
     */
    class log4cpp::Category& m_logger;
    
    /**
     * The locations helper
     */
    const Locations * m_locations;
    
    /**
     * The Singleton instance
     * @label singleton
     */
    static ServiceConfigurator * s_instance;
    
    /*
     * The command line arguments
     */
    static char ** s_argv;
    
    /*
     * The max service name length
     */
    static int s_nameLen;

    /** @link dependency 
     * @label configure*/
    /*# ComponentConfiguration lnkComponentConfiguration; */
};

} // End namespace config
} // End namespace glite

#endif //GLITE_DATA_CONFIG_SERVICE_SERVICECONFIGURATOR_HXX_
