/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  SysLog.h
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SysLog.hxx,v 1.5 2009/12/03 17:18:01 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_CONF_SERVICE_SYSLOG_HXX_
#define GLITE_DATA_CONF_SERVICE_SYSLOG_HXX_

#include <string>
#include <map>
#include <log4cpp/Priority.hh>
#include <unistd.h> 
#include <sys/types.h>

namespace log4cpp{
class CategoryStream;
}

namespace glite {
namespace config {

/**
 * EventNames
 * collection of descriptive strings of event names to be used in system logs.
 */
class EventNames{
public:
    /**
     * service was started
     */
    static const char * const ServiceStart;
    /**
     * service was re-configured
     */
    static const char * const ServiceReconfigure;
    /**
     * service was stopped
     */
    static const char * const ServiceStop;
    /**
     * service was finalized
     */
    static const char * const ServiceFinalize;
private:
    EventNames();
};
    
/**
 * SysLog
 * Creates system logs according to Middleware Security Audit Logging Guidelines
 */
class SysLog{
public:
    
    /**
     * The category used to log messages on the log file
     */
    static const std::string FileCategoryName;
    
    /**
     * The format string
     */
    static const std::string FormatString;
    
    /**
     * Set the session or request id
     * @param id [IN] the session or request id
     * @return a reference to the object itself
     */
    SysLog& id(const std::string& id);

    /**
     * Set the external request or session id
     * @param guid [IN] the external request or session id
     * @return a reference to the object itself
     */
    SysLog& guid(const std::string& guid);
        
    /**
     * Set the event name
     * @param event [IN] the event name
     * @return a reference to the object itself
     */
    SysLog& event(const std::string& event);
    
    /**
     * Set the event status to 0 or -1 (generic codes)
     * @param success [IN] success flag
     * @return a reference to the object itself
     */
    SysLog& success(bool success);

    /**
     * Set the event status to given value
     * @param status [IN] event status value: 
     *  < 0 -> failure
     *  >= 0 -> success
     * @return a reference to the object itself
     */
    SysLog& status(int status);
    
    /**
     * Set the message
     * @param message [IN] the message
     * @return a reference to the object itself
     */
    SysLog& msg(const std::string& message);
    
    /**
     * Add a generic name-value pair
     * @param name [IN] name
     * @param value [IN] value
     * @return a reference to the object itself
     */
    SysLog& add(const std::string& name,const std::string& value);
    
    /**
     * log a debug message
     */
    void debug(){
        log(log4cpp::Priority::DEBUG);
    }
    
    /**
     * log an informational message
     */
    void info(){
        log(log4cpp::Priority::INFO);
    }

    /**
     * log a notice message
     */
    void notice(){
        log(log4cpp::Priority::NOTICE);
    }
    
    /**
     * log a warning message
     */
    void warn(){
        log(log4cpp::Priority::WARN);
    }
    
    /**
     * log an error message
     */
    void error(){
        log(log4cpp::Priority::ERROR);
    }

protected:

    /**
     * Constructor
     * @param categoryName [IN] name of log4cpp category to use fos syslog
     * @param facility [IN] syslog facility as string (to log on file)
     * @param fileLog [IN] flag to determine if logging on file must be enabled
     */
    SysLog(
        const std::string&  categoryName,
        const std::string&  facility,
        bool                fileLog);
    
    /**
     * Destructor
     */
    ~SysLog();
        
private:
    
    /**
     * Log a message to the system log with given priority
     * @param priority [IN] the log priority
     */
    void log(log4cpp::Priority::PriorityLevel priority);
    
    /**
     * add_common
     * add common parts of log message
     * @param stream [IN] the log stream
     */
    void add_common(log4cpp::CategoryStream& stream);
    
    // name of logger category
    std::string m_categoryName;

    // facility as string (to log on file)
    std::string m_facility;
    // flag stating if message should also be logged on file
    bool m_fileLog;
    
    // session or request id (id=...)
    std::string m_id;

    // external request or session id (guid=...)
    std::string m_guid;
        
    // event name (event=...)
    std::string m_event;
    
    // event status code (status=...)
    int m_status;
    bool m_statusInit;
    
    // descriptive message (msg=...)
    std::string m_message;  
    
    // other name-value pairs
    typedef std::map<std::string,std::string> NameValuePairs;
    NameValuePairs m_nameValuePairs;
    
};

} // namespace config
} // namespace glite

#endif // GLITE_DATA_CONF_SERVICE_SYSLOG_HXX_


